@@ -8,7 +8,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -60,9 +60,9 @@ typedef struct _PredefinedAcpis {
 char *acpi_help[] = {
 	"",
 	N_("Make sure that the kernel module 'acerhk' is loaded.\n"
-	    "You can get it from http://www.informatik.hu-berlin.de/~tauber/acerhk/"),
+	    "You can get it from http://www.cakey.de/acerhk/"),
 	N_("Make sure that the kernel module 'acer_acpi' is loaded.\n"
-	    "You can get it from http://www.archernar.co.uk/acer_acpi/acer_acpi_main.html"),
+	    "You can get it from http://code.google.com/p/aceracpi/"),
 	N_("Make sure that the kernel module 'asus_laptop' is loaded."),
 	N_("Make sure that the kernel module 'asus_acpi' is loaded."),
 	N_("Make sure that the kernel module 'ibm_acpi' is loaded."),
@@ -634,9 +634,11 @@ static void acpi_prefs_save_func(PrefsPage * _page)
 		prefs_file_close_revert(pfile);
 		return;
 	}
-	fprintf(pfile->fp, "\n");
-
-	prefs_file_close(pfile);
+        if (fprintf(pfile->fp, "\n") < 0) {
+		FILE_OP_ERROR(rcpath, "fprintf");
+		prefs_file_close_revert(pfile);
+	} else
+	        prefs_file_close(pfile);
 }
 
 static void acpi_set(gboolean on)
@@ -840,6 +842,11 @@ gint plugin_init(gchar **error)
 
 gboolean plugin_done(void)
 {
+	if (blink_timeout_id)
+		g_source_remove(blink_timeout_id);
+	if (alertpanel_blink_timeout_id)
+		g_source_remove(alertpanel_blink_timeout_id);
+
 	acpi_done();
 	return TRUE;
 }
@@ -861,7 +868,7 @@ const gchar *plugin_type(void)
 
 const gchar *plugin_licence(void)
 {
-	return "GPL";
+	return "GPL3+";
 }
 
 const gchar *plugin_version(void)
