@@ -6,7 +6,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -405,7 +405,7 @@ static void rename_folder_cb(FolderView *folderview, guint action,
 	g_free(old_id);
 	g_free(new_id);
 
-	folder_item_prefs_save_config(item);
+	folder_item_prefs_save_config_recursive(item);
 	folder_write_list();
 }
 
