@@ -1,9 +1,10 @@
-/* Notification Plugin for Claws Mail
- * Copyright (C) 2005-2006 Holger Berndt
+/* Notification plugin for Claws-Mail
+ * Copyright (C) 2005-2007 Holger Berndt
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the version 2 of the GNU General Public License
- * as published by the Free Software Foundation.
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -11,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "pluginconfig.h"
@@ -24,7 +24,8 @@
 #include "mainwindow.h"
 #include "procmsg.h"
 #include "folder.h"
-#include "codeconv.h"
+#include "addrindex.h"
+#include "common/utils.h"
 #include "gtk/gtkutils.h"
 
 #include "gettext.h"
@@ -33,19 +34,12 @@
 #include "notification_prefs.h"
 #include "notification_foldercheck.h"
 #include "notification_pixbuf.h"
+#include "notification_core.h"
 
 #ifdef HAVE_LIBNOTIFY
 #  include <libnotify/notify.h>
 #endif
 
-typedef enum {
-  F_TYPE_MAIL=0,
-  F_TYPE_NEWS,
-  F_TYPE_CALENDAR,
-  F_TYPE_RSS,
-  F_TYPE_LAST
-} NotificationFolderType;
-
 typedef struct {
   gint count;
   guint timeout_id;
@@ -79,7 +73,6 @@ static gboolean notification_libnotify_create(MsgInfo*,
 static gboolean notification_libnotify_add_msg(MsgInfo*,
 					       NotificationFolderType);
 static void default_action_cb(NotifyNotification*, const char*,void*);
-static gchar* notification_libnotify_sanitize_str(gchar*);
 static void notification_libnotify_free_func(gpointer);
 
 #else /* !HAVE_LIBNOTIFY */
@@ -205,6 +198,7 @@ static gboolean popup_timeout_fun(gpointer data)
   G_LOCK(popup);
 #ifdef HAVE_LIBNOTIFY
   ppopup = &(popup[nftype]);
+
   if(!notify_notification_close(ppopup->notification, &(ppopup->error))) {
     debug_print("Notification Plugin: Failed to close notification: %s.\n",
 		ppopup->error->message);
@@ -278,6 +272,7 @@ static gboolean notification_libnotify_create(MsgInfo *msginfo,
   gchar *utf8_str = NULL;
   gchar *subj = NULL;
   gchar *from = NULL;
+  gchar *foldname = NULL;
 
   g_return_val_if_fail(msginfo, FALSE);
 
@@ -299,32 +294,24 @@ static gboolean notification_libnotify_create(MsgInfo *msginfo,
                                                   msginfo->from : _("(No From)"));
     subj    = notification_libnotify_sanitize_str(msginfo->subject ?
                                                   msginfo->subject : _("(No Subject)"));
-    text    = g_strconcat(from,"\n\n",subj,NULL);
+	if (notify_config.popup_display_folder_name) {
+		foldname = notification_libnotify_sanitize_str(msginfo->folder->path);
+    	text = g_strconcat(from,"\n\n", subj, "\n\n", foldname, NULL);
+	}
+	else
+		text = g_strconcat(from, "\n\n",subj, NULL);
+
     /* Make sure text is valid UTF8 */
-    if(!g_utf8_validate(text, -1, NULL)) {
-      debug_print("Notification plugin: String is not valid utf8, trying to fix it...\n");
-      /* fix it */
-      utf8_str = conv_codeset_strdup(text,
-				     conv_get_locale_charset_str_no_utf8(),
-				     CS_INTERNAL);
-      /* check if the fix worked */
-      if(utf8_str == NULL || !g_utf8_validate(utf8_str, -1, NULL)) {
-	debug_print("Notification plugin: String is still not valid utf8, sanitizing...\n");
-	utf8_str = g_malloc(strlen(text)*2+1);
-	conv_localetodisp(utf8_str, strlen(text)*2+1, text);
-      }
-      g_free(text);
-    }
-    else {
-      debug_print("Notification plugin: String is valid utf8\n");
-      utf8_str = text;
-    }
+    utf8_str = notification_validate_utf8_str(text);
+    g_free(text);
+
     if(from) g_free(from);
     if(subj) g_free(subj);
+    if(foldname) g_free(foldname);
     break;
   case F_TYPE_NEWS:
-    summary = _("New News message");
-    utf8_str    = g_strdup(_("A new mesesage arrived"));
+    summary = _("New News post");
+    utf8_str    = g_strdup(_("A new message arrived"));
     break;
   case F_TYPE_CALENDAR:
     summary = _("New Calendar message");
@@ -356,9 +343,38 @@ static gboolean notification_libnotify_create(MsgInfo *msginfo,
 				 notification_libnotify_free_func);
 
   /* Icon */
-  pixbuf = notification_pixbuf_get_logo_64x64();
-  if(pixbuf)
+  pixbuf = NULL;
+  if(msginfo && msginfo->from) {
+    gchar *icon_path;
+    icon_path = addrindex_get_picture_file(msginfo->from);
+    if(is_file_exist(icon_path)) {
+      GError *error = NULL;
+      gint w, h;
+
+      gdk_pixbuf_get_file_info(icon_path, &w, &h);
+      if((w > 64) || (h > 64))
+	pixbuf = gdk_pixbuf_new_from_file_at_scale(icon_path,
+						   64, 64, TRUE, &error);
+      else
+	pixbuf = gdk_pixbuf_new_from_file(icon_path, &error);
+
+      if(!pixbuf) {
+	debug_print("Could not load picture file: %s\n",
+		    error ? error->message : "no details");
+	g_error_free(error);
+      }
+    }
+    else 
+      debug_print("Picture path does not exist: %s\n",icon_path);
+    g_free(icon_path);
+  }
+  if(!pixbuf)
+   pixbuf = g_object_ref(notification_pixbuf_get(NOTIFICATION_CM_LOGO_64x64));
+
+  if(pixbuf) {
     notify_notification_set_icon_from_pixbuf(ppopup->notification, pixbuf);
+    g_object_unref(pixbuf);
+  }
   else /* This is not fatal */
     debug_print("Notification plugin: Icon could not be loaded.\n");
 
@@ -404,6 +420,7 @@ static gboolean notification_libnotify_add_msg(MsgInfo *msginfo,
   gchar *text;
   gboolean retval;
   NotificationPopup *ppopup;
+  GdkPixbuf *pixbuf;
 
   ppopup = &(popup[nftype]);
 
@@ -417,6 +434,11 @@ static gboolean notification_libnotify_add_msg(MsgInfo *msginfo,
     ppopup->msg_path = NULL;
   }
 
+  /* make sure we show a logo on many msg arrival */
+  pixbuf = notification_pixbuf_get(NOTIFICATION_CM_LOGO_64x64);
+  if(pixbuf)
+    notify_notification_set_icon_from_pixbuf(ppopup->notification, pixbuf);
+
   switch(nftype) {
   case F_TYPE_MAIL:
     summary = _("Mail message");
@@ -469,41 +491,6 @@ static gboolean notification_libnotify_add_msg(MsgInfo *msginfo,
   return TRUE;
 }
 
-#define STR_MAX_LEN 511
-/* Returns a newly allocated string which needs to be freed */
-static gchar* notification_libnotify_sanitize_str(gchar *in)
-{
-  gint i_out;
-  gchar tmp_str[STR_MAX_LEN+1];
-
-  if(in == NULL) return NULL;
-
-  i_out = 0;
-  while(*in) {
-    if(*in == '<') {
-      if(i_out+3 >= STR_MAX_LEN) break;
-      memcpy(&(tmp_str[i_out]),"&lt;",4);
-      in++; i_out += 4;
-    }
-    else if(*in == '>') {
-      if(i_out+3 >= STR_MAX_LEN) break;
-      memcpy(&(tmp_str[i_out]),"&gt;",4);
-      in++; i_out += 4;
-    }
-    else if(*in == '&') {
-      if(i_out+4 >= STR_MAX_LEN) break;
-      memcpy(&(tmp_str[i_out]),"&amp;",5);
-      in++; i_out += 5;
-    }
-    else {
-      if(i_out >= STR_MAX_LEN) break;
-      tmp_str[i_out++] = *in++;
-    }
-  }
-  tmp_str[i_out] = '\0';
-  return strdup(tmp_str);
-}
-
 void notification_libnotify_free_func(gpointer data)
 {
   if(popup[F_TYPE_MAIL].msg_path) {
