@@ -6,7 +6,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  *(at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -20,13 +20,14 @@
  */
 
 #include "poppler_viewer.h"
+#include "printing.h"
+#include "prefs_common.h"
 
 static FileType pdf_viewer_mimepart_get_type(MimeInfo *partinfo);
 static MimeViewerFactory pdf_viewer_factory;
 
 static void pdf_viewer_show_mimepart(MimeViewer *_viewer, const gchar *infile,
 				MimeInfo *partinfo);
-static gint pdf_viewer_show_mimepart_real(MimeViewer *_viewer);
 
 static MimeViewer *pdf_viewer_create(void);
 static void pdf_viewer_clear(MimeViewer *_viewer);
@@ -54,18 +55,20 @@ static void pdf_viewer_index_row_activated(GtkTreeView		*tree_view,
 
 static GtkTable * pdf_viewer_fill_info_table(PdfViewer *viewer);
 
-
 /* Callbacks */
+static void pdf_viewer_move_events_cb(GtkWidget *widget, GdkEventMotion *event, PdfViewer *viewer); 
+static void pdf_viewer_button_press_events_cb(GtkWidget *widget, GdkEventButton *event, PdfViewer *viewer); 
+static void pdf_viewer_mouse_scroll_destroy_cb(GtkWidget *widget, GdkEventButton *event, PdfViewer *viewer); 
 static void pdf_viewer_button_first_page_cb(GtkButton *button, PdfViewer *viewer);
 static void pdf_viewer_button_last_page_cb(GtkButton *button, PdfViewer *viewer);
-static void pdf_viewer_spin_change_page_cb(GtkSpinButton *button, PdfViewer *viewer);
 static void pdf_viewer_button_zoom_in_cb(GtkButton *button, PdfViewer *viewer);
 static void pdf_viewer_button_zoom_out_cb(GtkButton *button, PdfViewer *viewer);
 static void pdf_viewer_button_zoom_fit_cb(GtkButton *button, PdfViewer *viewer);
-static void pdf_viewer_spin_zoom_scroll_cb(GtkSpinButton *button, PdfViewer *viewer);
 static void pdf_viewer_button_zoom_width_cb(GtkButton *button, PdfViewer *viewer);
 static void pdf_viewer_button_rotate_right_cb(GtkButton *button, PdfViewer *viewer);
 static void pdf_viewer_button_rotate_left_cb(GtkButton *button, PdfViewer *viewer);
+static void pdf_viewer_spin_change_page_cb(GtkSpinButton *button, PdfViewer *viewer);
+static void pdf_viewer_spin_zoom_scroll_cb(GtkSpinButton *button, PdfViewer *viewer);
 /* Show/Hide the index pane */
 static void pdf_viewer_show_document_index_cb(GtkButton *button, PdfViewer *viewer);
 static void pdf_viewer_button_document_info_cb(GtkButton *button, PdfViewer *viewer);
@@ -299,9 +302,7 @@ static gboolean	pdf_viewer_text_search(MimeViewer *_viewer, gboolean backward,
 		
 		for(i = 1; i <= viewer->num_pages; i++) {
 			
-			gchar *page_str = g_strdup_printf("%d",i);
-			PopplerPage *pdf_page = poppler_document_get_page_by_label(viewer->pdf_doc, page_str);
-			g_free(page_str);
+			PopplerPage *pdf_page = poppler_document_get_page(viewer->pdf_doc, i - 1);
 			viewer->page_results = poppler_page_find_text(pdf_page, str);
 			
 			if (viewer->page_results != NULL) {
@@ -668,26 +669,123 @@ static void pdf_viewer_button_zoom_out_cb(GtkButton *button, PdfViewer *viewer)
 	gtk_spin_button_spin(GTK_SPIN_BUTTON(viewer->zoom_scroll), GTK_SPIN_STEP_BACKWARD, ZOOM_FACTOR);
 	
 }
-/* Set the grab cursor*/
-static void pdf_viewer_init_mouse_scroll_cb(GtkWidget *widget, GdkEventButton *event, PdfViewer *viewer) 
+
+static void pdf_viewer_button_press_events_cb(GtkWidget *widget, GdkEventButton *event, PdfViewer *viewer) 
 {
+	gchar *uri;
+	#ifdef HAVE_POPPLER_DEST_NAMED
+	PopplerDest *dest;
+	#endif
 	static GdkCursor *hand_cur = NULL;
 	
 	if (!hand_cur) hand_cur = gdk_cursor_new(GDK_FLEUR);
-	
-	if (event->button == 1) {
+
+	/* Execute Poppler Links */
+	if (event->button == 1 && viewer->in_link) {
+		switch (viewer->link_action->type) {
+		case POPPLER_ACTION_UNKNOWN:
+			debug_print("action unknown\n");
+			break;
+		case POPPLER_ACTION_GOTO_DEST:
+			if (viewer->link_action->goto_dest.dest->type == POPPLER_DEST_XYZ || 
+					viewer->link_action->goto_dest.dest->type == POPPLER_DEST_FITH) {
+				gtk_spin_button_set_value(GTK_SPIN_BUTTON(viewer->cur_page), 
+									  (gdouble)viewer->link_action->goto_dest.dest->page_num);
+			}
+		#ifdef HAVE_POPPLER_DEST_NAMED
+			else if (viewer->link_action->goto_dest.dest->type == POPPLER_DEST_NAMED) {
+				dest = poppler_document_find_dest(
+					viewer->pdf_doc, viewer->link_action->goto_dest.dest->named_dest);
+			if (dest->type != POPPLER_DEST_XYZ) {
+				g_warning("couldn't figure out link\n");
+				poppler_dest_free(dest);
+				break;
+			}
+			gtk_spin_button_set_value(GTK_SPIN_BUTTON(viewer->cur_page), 
+									  (gdouble)dest->page_num);
+			poppler_dest_free(dest);
+		} 
+		#endif
+			break;
+		case POPPLER_ACTION_GOTO_REMOTE:
+			#ifdef HAVE_POPPLER_DEST_NAMED
+			dest = poppler_document_find_dest(
+					viewer->pdf_doc, viewer->link_action->goto_remote.dest->named_dest);
+			if (dest->type != POPPLER_DEST_XYZ) {
+				g_warning ("couldn't figure out link\n");
+				poppler_dest_free(dest);
+				break;
+			}
+			gtk_spin_button_set_value(GTK_SPIN_BUTTON(viewer->cur_page),
+									  (gdouble)dest->page_num);
+			poppler_dest_free(dest);
+			#endif
+			break;
+		case POPPLER_ACTION_LAUNCH:
+			debug_print("action launch not yet implemented\n");
+			break;
+		case POPPLER_ACTION_URI:
+			uri = g_strdup(viewer->link_action->uri.uri);
+			if (!g_ascii_strncasecmp(uri, "mailto:", 7)) 
+				compose_new(NULL, uri + 7, NULL);
+			else 
+				open_uri(uri, prefs_common_get_uri_cmd());
+			g_free(uri);
+			break;
+		case POPPLER_ACTION_NAMED:
+			debug_print("action named not yet implemented\n");
+			break;
+		case POPPLER_ACTION_MOVIE:
+			debug_print("yoyoyo ;-) a movie?\n");
+			break;
+		}
+		if (((MimeViewer *)viewer)->mimeview && 
+			((MimeViewer *)viewer)->mimeview->messageview && 
+			((MimeViewer *)viewer)->mimeview->messageview->window && 
+			((MimeViewer *)viewer)->mimeview->messageview->window->window) 
+			gdk_window_set_cursor (((MimeViewer *)viewer)->mimeview->messageview->window->window, NULL);
+		else
+			gdk_window_set_cursor (mainwindow_get_mainwindow()->window->window, NULL);
+	}
+
+	/* Init document to be scrolled with left mouse click */
+	if (event->button == 1 && !viewer->in_link) {
 		viewer->pdf_view_scroll = TRUE;
-		gdk_window_set_cursor (mainwindow_get_mainwindow()->window->window,hand_cur);
+		if (((MimeViewer *)viewer)->mimeview && 
+			((MimeViewer *)viewer)->mimeview->messageview && 
+			((MimeViewer *)viewer)->mimeview->messageview->window && 
+			((MimeViewer *)viewer)->mimeview->messageview->window->window) 
+			gdk_window_set_cursor (((MimeViewer *)viewer)->mimeview->messageview->window->window, hand_cur);
+		else
+			gdk_window_set_cursor (mainwindow_get_mainwindow()->window->window, hand_cur);
+
 		viewer->last_x = event->x;
 		viewer->last_y = event->y;
 		viewer->last_dir_x = 0;
 		viewer->last_dir_y = 0;
 	}
 }
-/* Grab the document and scroll it with mouse*/
-static void pdf_viewer_mouse_scroll_cb(GtkWidget *widget, GdkEventMotion *event, PdfViewer *viewer) 
+/* Set the normal cursor*/
+static void pdf_viewer_mouse_scroll_destroy_cb(GtkWidget *widget, GdkEventButton *event, PdfViewer *viewer) 
 {
+	
+	if (event->button == 1) {
+		viewer->pdf_view_scroll = FALSE;
+		if (((MimeViewer *)viewer)->mimeview && 
+			((MimeViewer *)viewer)->mimeview->messageview && 
+			((MimeViewer *)viewer)->mimeview->messageview->window && 
+			((MimeViewer *)viewer)->mimeview->messageview->window->window) 
+			gdk_window_set_cursor (((MimeViewer *)viewer)->mimeview->messageview->window->window, NULL);
+		else
+			gdk_window_set_cursor (mainwindow_get_mainwindow()->window->window, NULL);
+	}
+}
+
+static void pdf_viewer_move_events_cb(GtkWidget *widget, GdkEventMotion *event, PdfViewer *viewer) 
+{
+	/* Grab the document and scroll it with mouse */ 
 	if (viewer->pdf_view_scroll) {
+
 		viewer->pdf_view_vadj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(viewer->scrollwin));
 		viewer->pdf_view_hadj = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(viewer->scrollwin));
 			
@@ -700,7 +798,7 @@ static void pdf_viewer_mouse_scroll_cb(GtkWidget *widget, GdkEventMotion *event,
 				}
 				viewer->last_dir_x = -1;
 			}
-			else if(event->x > viewer->last_x
+			else if (event->x > viewer->last_x
 					&& viewer->pdf_view_hadj->value > 0.0)  {
 				if (viewer->last_dir_x == +1) {
 					viewer->pdf_view_hadj->value += viewer->last_x - event->x; 
@@ -719,7 +817,7 @@ static void pdf_viewer_mouse_scroll_cb(GtkWidget *widget, GdkEventMotion *event,
 				}
 				viewer->last_dir_y = -1;
 			}
-			else if(event->y > viewer->last_y
+			else if (event->y > viewer->last_y
 					&& viewer->pdf_view_vadj->value > 0.0)  {
 				if (viewer->last_dir_y == +1) {
 					viewer->pdf_view_vadj->value += viewer->last_y - event->y; 
@@ -732,18 +830,93 @@ static void pdf_viewer_mouse_scroll_cb(GtkWidget *widget, GdkEventMotion *event,
 			viewer->last_y = event->y;
 			GTK_EVENTS_FLUSH();
 		} 
-}
-/* Set the normal cursor*/
-static void pdf_viewer_destroy_mouse_scroll_cb(GtkWidget *widget, GdkEventButton *event, PdfViewer *viewer) 
-{
+	else {	
+	/* Link Mapping */
+	static GList *l;
+	static GdkCursor *link_cur = NULL;
+	static GtkRequisition size;
+	static gdouble x,y, x1, y1, x2, y2;
+	gboolean ccur;
 	
-	if (event->button == 1) {
-		viewer->pdf_view_scroll = FALSE;
-		gdk_window_set_cursor (mainwindow_get_mainwindow()->window->window, 
-							  NULL);
+	viewer->pdf_view_vadj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(viewer->scrollwin));
+	viewer->pdf_view_hadj = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(viewer->scrollwin));
+	
+	if (!link_cur) link_cur = gdk_cursor_new(GDK_HAND2);
+
+	ccur = FALSE;
+	viewer->in_link = FALSE;	
+	for (l = viewer->link_map; l; l = g_list_next (l)) {
+		PopplerLinkMapping *lmapping;
+		lmapping = (PopplerLinkMapping *)l->data;
+
+		x1 = lmapping->area.x1;
+		y1 = lmapping->area.y1;
+		x2 = lmapping->area.x2;
+		y2 = lmapping->area.y2;
+		gtk_widget_size_request(viewer->pdf_view, &size);
+		
+		switch (viewer->rotate) {
+		case 0:
+		case 360:
+				if (size.width != viewer->pdf_view_hadj->upper)
+					x = (event->x - (viewer->pdf_view_hadj->upper - size.width) / 2) / viewer->zoom;
+				else
+					x = event->x / viewer->zoom;
+
+				y = (viewer->pdf_view_vadj->upper - event->y) / viewer->zoom;
+			break;
+		case 90:
+				if (size.width != viewer->pdf_view_hadj->upper)
+					y = (event->x - (viewer->pdf_view_hadj->upper - size.width) / 2) / viewer->zoom;
+				else
+					y = event->x / viewer->zoom;
+
+				x = (event->y) / viewer->zoom;
+			break;
+		case 180:
+				if (size.width != viewer->pdf_view_hadj->upper)
+					x = ((viewer->pdf_view_hadj->upper -  event->x) - ((viewer->pdf_view_hadj->upper - size.width) / 2)) / viewer->zoom;
+				else
+					x =  ((viewer->pdf_view_hadj->upper -  event->x) - (viewer->pdf_view_hadj->upper - size.width)) / viewer->zoom;
+
+				y = (event->y) / viewer->zoom;
+			break;
+		case 270:
+				if (size.width != viewer->pdf_view_hadj->upper)
+					y = ((viewer->pdf_view_hadj->upper -  event->x) - ((viewer->pdf_view_hadj->upper - size.width) / 2)) / viewer->zoom;
+				else
+					y =  ((viewer->pdf_view_hadj->upper -  event->x) - (viewer->pdf_view_hadj->upper - size.width)) / viewer->zoom;
+
+				x = (viewer->pdf_view_vadj->upper - event->y) / viewer->zoom;
+			break;
+		}
+
+		if ( (x > x1 && x < x2) && (y > y1 && y < y2) ) {
+				viewer->in_link = TRUE;
+			if (((MimeViewer *)viewer)->mimeview && 
+				((MimeViewer *)viewer)->mimeview->messageview && 
+				((MimeViewer *)viewer)->mimeview->messageview->window && 
+				((MimeViewer *)viewer)->mimeview->messageview->window->window) 
+					gdk_window_set_cursor (((MimeViewer *)viewer)->mimeview->messageview->window->window, link_cur);
+				else
+					gdk_window_set_cursor (mainwindow_get_mainwindow()->window->window, link_cur);
+				
+				viewer->link_action = lmapping->action; 
+				ccur = TRUE;
+		}
+		if (!ccur) {
+			if (((MimeViewer *)viewer)->mimeview && 
+				((MimeViewer *)viewer)->mimeview->messageview && 
+				((MimeViewer *)viewer)->mimeview->messageview->window && 
+				((MimeViewer *)viewer)->mimeview->messageview->window->window) 
+				gdk_window_set_cursor (((MimeViewer *)viewer)->mimeview->messageview->window->window, NULL);
+			else
+				gdk_window_set_cursor (mainwindow_get_mainwindow()->window->window, NULL);
+		}
+	}
+	g_free(l);
 	}
 }
-
 static gboolean pdf_viewer_scroll_cb(GtkWidget *widget, GdkEventScroll *event,
 				    PdfViewer *viewer)
 {
@@ -809,10 +982,9 @@ static void pdf_viewer_button_rotate_right_cb(GtkButton *button, PdfViewer *view
 	if (viewer->rotate == 360) {
 		viewer->rotate = 0;
 	}
-
+	
 	viewer->rotate += (gint) ROTATION;
-	pdf_viewer_update((MimeViewer *)viewer,
-		FALSE,
+	pdf_viewer_update((MimeViewer *)viewer, FALSE,
 		gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(viewer->cur_page)));
 }
 
@@ -821,10 +993,9 @@ static void pdf_viewer_button_rotate_left_cb(GtkButton *button, PdfViewer *viewe
 	if (viewer->rotate == 0) {
 		viewer->rotate = 360;
 	}
-
+	
 	viewer->rotate = abs(viewer->rotate -(gint) ROTATION);
-	pdf_viewer_update((MimeViewer *)viewer,
-		FALSE,
+	pdf_viewer_update((MimeViewer *)viewer, FALSE,
 		gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(viewer->cur_page)));
 }
 
@@ -974,7 +1145,7 @@ static void pdf_viewer_update(MimeViewer *_viewer, gboolean reload_file, int pag
 				tmpfile, viewer->filename);
 			result = execute_command_line(cmdline, FALSE);
 			if (result == 0) {
-				tmp = g_strdup_printf("file://%s", tmpfile);
+				tmp = g_filename_to_uri(tmpfile, NULL, NULL);
 				viewer->pdf_doc = poppler_document_new_from_file( tmp, NULL, &error);
 				g_free(tmp);
 			} 
@@ -1012,10 +1183,10 @@ static void pdf_viewer_update(MimeViewer *_viewer, gboolean reload_file, int pag
 				tmp,
 				NULL);
 		g_free(tmp);
-		gtk_label_set_text(GTK_LABEL(viewer->doc_label),
-				    	(tmp = g_strdup_printf(_("(%d page%s)"), 
-							viewer->num_pages,
-							viewer->num_pages > 1 ? "s":"")));
+
+		tmp = g_strdup_printf(ngettext("%d page", "%d pages", viewer->num_pages),
+				viewer->num_pages);
+		gtk_label_set_text(GTK_LABEL(viewer->doc_label), tmp);
 		g_free(tmp);
 
 		pdf_viewer_show_controls(viewer, TRUE);
@@ -1037,20 +1208,36 @@ static void pdf_viewer_update(MimeViewer *_viewer, gboolean reload_file, int pag
 		return;
 	}
 
+	if (page_num == 1) { 
+		gtk_widget_set_sensitive(viewer->first_page, FALSE);
+		gtk_widget_set_sensitive(viewer->prev_page, FALSE);
+	}
+	else {
+		gtk_widget_set_sensitive(viewer->first_page, TRUE);
+		gtk_widget_set_sensitive(viewer->prev_page, TRUE);
+	}
+
+	if (page_num == viewer->num_pages) { 
+		gtk_widget_set_sensitive(viewer->last_page, FALSE);
+		gtk_widget_set_sensitive(viewer->next_page, FALSE);
+	}
+	else {
+		gtk_widget_set_sensitive(viewer->last_page, TRUE);
+		gtk_widget_set_sensitive(viewer->next_page, TRUE);
+	}
+
 	/* check for the index if exists */
 	pdf_viewer_set_index_button_sensitive((PdfViewer *) viewer);
 	
 	if (page_num > 0 && page_num <= viewer->num_pages) {
 
-		gchar *page_str = g_strdup_printf("%d", page_num);
 		GTK_EVENTS_FLUSH();
 		
 		if (viewer->pdf_page) {
 			g_object_unref(G_OBJECT(viewer->pdf_page));
 		}
 			
-		viewer->pdf_page = poppler_document_get_page_by_label(viewer->pdf_doc, page_str);
-		g_free(page_str);
+		viewer->pdf_page = poppler_document_get_page(viewer->pdf_doc, page_num - 1);
 
 		if (viewer->pdf_page == NULL) {
 			g_warning("Page not found\n");
@@ -1072,10 +1259,19 @@ static void pdf_viewer_update(MimeViewer *_viewer, gboolean reload_file, int pag
 									viewer->height, viewer->zoom, viewer->rotate);
 
 		}
+	
+	/* Get Links Mapping */
+	if (viewer->link_map) {
+		poppler_page_free_link_mapping(viewer->link_map);
+	}
+	viewer->link_map = poppler_page_get_link_mapping(viewer->pdf_page);
+	
 	}
 }
 
-static gint pdf_viewer_show_mimepart_real(MimeViewer *_viewer)
+
+static void pdf_viewer_show_mimepart(MimeViewer *_viewer, const gchar *infile,
+				MimeInfo *partinfo)
 {
 	PdfViewer *viewer = (PdfViewer *) _viewer;
 	gchar buf[4096];
@@ -1083,7 +1279,9 @@ static gint pdf_viewer_show_mimepart_real(MimeViewer *_viewer)
 	MessageView *messageview = ((MimeViewer *)viewer)->mimeview 
 					?((MimeViewer *)viewer)->mimeview->messageview 
 					: NULL;
-	MimeInfo *partinfo = viewer->to_load;
+
+	viewer->rotate = 0;
+	viewer->to_load = partinfo;
 
 	memset(buf, 0, sizeof(buf));
 	messageview->updating = TRUE;
@@ -1100,7 +1298,7 @@ static gint pdf_viewer_show_mimepart_real(MimeViewer *_viewer)
 	if (partinfo) {
 		viewer->target_filename = procmime_get_part_file_name(partinfo);
 		viewer->filename = procmime_get_tmp_file_name(partinfo);
-		viewer->fsname = g_strconcat("file://", viewer->filename, NULL);
+		viewer->fsname = g_filename_to_uri(viewer->filename, NULL, NULL);
 	}
 	
 	if (partinfo && !(procmime_get_part(viewer->filename, partinfo) < 0)) {
@@ -1124,17 +1322,6 @@ static gint pdf_viewer_show_mimepart_real(MimeViewer *_viewer)
 	pdf_viewer_update((MimeViewer *)viewer, TRUE, 1);
 	
 	messageview->updating = FALSE;
-	return FALSE;
-}
-
-static void pdf_viewer_show_mimepart(MimeViewer *_viewer, const gchar *infile,
-				MimeInfo *partinfo)
-{
-	PdfViewer *viewer = (PdfViewer *) _viewer;
-	viewer->rotate = 0;
-	viewer->to_load = partinfo;
-	gtk_timeout_add(5,(GtkFunction)pdf_viewer_show_mimepart_real, viewer);
-	
 }
 
 static void pdf_viewer_clear(MimeViewer *_viewer)
@@ -1152,7 +1339,7 @@ static void pdf_viewer_clear(MimeViewer *_viewer)
 		g_object_unref(G_OBJECT(viewer->pdf_doc));
 		viewer->pdf_doc = NULL;
 	}
-
+	
 	vadj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(viewer->scrollwin));
 	vadj->value = 0.0;
 	g_signal_emit_by_name(G_OBJECT(vadj), "value-changed", 0);
@@ -1171,6 +1358,7 @@ static void pdf_viewer_destroy(MimeViewer *_viewer)
 	
 	if (viewer->pdf_index) poppler_index_iter_free(viewer->pdf_index);
 
+	poppler_page_free_link_mapping (viewer->link_map);
 	gtk_widget_unref(GTK_WIDGET(viewer->vbox));
 	gtk_widget_unref(GTK_WIDGET(viewer->pdf_view));
 	gtk_widget_unref(GTK_WIDGET(viewer->doc_index_pane));
@@ -1262,6 +1450,76 @@ static void button_set_pixmap(GtkWidget *widg, char **button_image)
 	gtk_table_set_col_spacing(GTK_TABLE(viewer->buttons_table), col, 3*BUTTON_H_PADDING); \
 	col++;
 
+#if GTK_CHECK_VERSION(2,10,0) && POPPLER_HAS_CAIRO && !USE_LIBGNOMEPRINT
+static PangoContext *pdf_viewer_get_pango_context(gpointer data)
+{
+	return NULL;
+}
+
+static gpointer pdf_viewer_get_data_to_print(gpointer data, gint sel_start, gint sel_end)
+{
+	return NULL; /* we don't need it */
+}
+
+static void pdf_viewer_cb_begin_print(GtkPrintOperation *op, GtkPrintContext *context,
+			   gpointer user_data)
+{
+  PrintData *print_data;
+  PopplerDocument *pdf_doc;
+  gint n_pages = 0;
+  print_data = (PrintData*) user_data;
+  pdf_doc = (PopplerDocument *)printing_get_renderer_data(print_data);
+
+  debug_print("Preparing print job...\n");
+
+  n_pages = poppler_document_get_n_pages(pdf_doc);
+  printing_set_n_pages(print_data, n_pages);
+  gtk_print_operation_set_n_pages(op, n_pages);
+
+  debug_print("Starting print job...\n");
+}
+
+static void pdf_viewer_cb_draw_page(GtkPrintOperation *op, GtkPrintContext *context,
+			 int page_nr, gpointer user_data)
+{
+  cairo_t *cr;
+  PrintData *print_data;
+  PopplerDocument *pdf_doc;
+  PopplerPage *pdf_page;
+  
+  print_data = (PrintData*) user_data;
+  pdf_doc = (PopplerDocument *)printing_get_renderer_data(print_data);
+  pdf_page = poppler_document_get_page(pdf_doc, page_nr);
+
+  cr = gtk_print_context_get_cairo_context(context);
+  cairo_scale(cr, printing_get_zoom(print_data), printing_get_zoom(print_data));
+  cairo_set_source_rgb(cr, 0., 0., 0.);
+
+  poppler_page_render(pdf_page, cr);
+
+  g_object_unref(G_OBJECT(pdf_page));
+
+  debug_print("Sent page %d to printer\n", page_nr+1);
+}
+
+static void pdf_viewer_print(MimeViewer *mviewer)
+{
+	PdfViewer *viewer = (PdfViewer *)mviewer;
+	PrintRenderer *pdf_renderer = g_new0(PrintRenderer, 1);
+	MainWindow *mainwin = mainwindow_get_mainwindow();
+	
+	pdf_renderer->get_pango_context = pdf_viewer_get_pango_context;
+	pdf_renderer->get_data_to_print = pdf_viewer_get_data_to_print;
+	pdf_renderer->cb_begin_print    = pdf_viewer_cb_begin_print;
+	pdf_renderer->cb_draw_page      = pdf_viewer_cb_draw_page;
+
+	printing_print_full(mainwin ? GTK_WINDOW(mainwin->window):NULL, 
+			pdf_renderer, viewer->pdf_doc, -1, -1);
+	
+	g_free(pdf_renderer);
+}
+#endif
+
 static MimeViewer *pdf_viewer_create(void)
 {
 	PdfViewer *viewer;
@@ -1285,6 +1543,9 @@ static MimeViewer *pdf_viewer_create(void)
 	viewer->mimeviewer.text_search = pdf_viewer_text_search;
 	viewer->mimeviewer.scroll_page = pdf_viewer_scroll_page;
 	viewer->mimeviewer.scroll_one_line = pdf_viewer_scroll_one_line;
+#if GTK_CHECK_VERSION(2,10,0) && POPPLER_HAS_CAIRO && !USE_LIBGNOMEPRINT
+	viewer->mimeviewer.print = pdf_viewer_print;
+#endif
 	viewer->scrollwin = gtk_scrolled_window_new(NULL, NULL);
 	viewer->scrollwin_index = gtk_scrolled_window_new(NULL, NULL);
 	viewer->pdf_view_ebox = gtk_event_box_new();
@@ -1293,8 +1554,9 @@ static MimeViewer *pdf_viewer_create(void)
 	viewer->mimeinfo  = NULL;
 
 	viewer->pdf_view = gtk_image_new();
-	gtk_widget_set_events(viewer->pdf_view,
+	gtk_widget_set_events(viewer->pdf_view_ebox,
 						GDK_BUTTON_RELEASE_MASK
+						| GDK_POINTER_MOTION_MASK
 						| GDK_BUTTON_PRESS_MASK
 						| GDK_BUTTON_MOTION_MASK
 					    );
@@ -1634,15 +1896,15 @@ static MimeViewer *pdf_viewer_create(void)
 				   (gpointer) viewer);
 	g_signal_connect(G_OBJECT(viewer->pdf_view_ebox), 
 				    "button_press_event", 
-				    G_CALLBACK(pdf_viewer_init_mouse_scroll_cb), 
+				    G_CALLBACK(pdf_viewer_button_press_events_cb), 
 				   (gpointer) viewer);
 	g_signal_connect(G_OBJECT(viewer->pdf_view_ebox), 
 				    "button_release_event", 
-				    G_CALLBACK(pdf_viewer_destroy_mouse_scroll_cb), 
+				    G_CALLBACK(pdf_viewer_mouse_scroll_destroy_cb), 
 				   (gpointer) viewer);
 	g_signal_connect(G_OBJECT(viewer->pdf_view_ebox), 
 				    "motion_notify_event", 
-				    G_CALLBACK(pdf_viewer_mouse_scroll_cb), 
+				    G_CALLBACK(pdf_viewer_move_events_cb), 
 				   (gpointer) viewer);
 
 	viewer->target_filename = NULL;
@@ -1705,7 +1967,7 @@ const gchar *plugin_type(void)
 
 const gchar *plugin_licence(void)
 {
-	return "GPL";
+	return "GPL3+";
 }
 
 const gchar *plugin_version(void)
