@@ -35,7 +35,6 @@
 #include <pthread.h>
 #include <time.h>
 
-#include "common/utils.h"
 #include "common/claws.h"
 #include "codeconv.h"
 #include "procmsg.h"
@@ -48,6 +47,7 @@
 #include "prefs_common.h"
 #include "defs.h"
 #include "inc.h"
+#include "common/utils.h"
 
 #include "gettext.h"
 
@@ -257,6 +257,7 @@ xmlDocPtr rssyl_fetch_feed(const gchar *url, time_t last_update, gchar **title,
 
 #ifdef USE_PTHREAD
 	pthread_t pt;
+	pthread_attr_t pta;
 #endif
 	gchar *msg = NULL, *tmptitle = NULL;
 	gchar *content;
@@ -280,7 +281,9 @@ xmlDocPtr rssyl_fetch_feed(const gchar *url, time_t last_update, gchar **title,
 	GTK_EVENTS_FLUSH();
 
 #ifdef USE_PTHREAD
-	if( pthread_create(&pt, PTHREAD_CREATE_JOINABLE, rssyl_fetch_feed_threaded,
+	if (pthread_attr_init(&pta) != 0 ||
+	    pthread_attr_setdetachstate(&pta, PTHREAD_CREATE_JOINABLE) != 0 ||
+	    pthread_create(&pt, &pta, rssyl_fetch_feed_threaded,
 				(void *)ctx) != 0 ) {
 		/* Bummer, couldn't create thread. Continue non-threaded */
 		template = rssyl_fetch_feed_threaded(ctx);
@@ -335,6 +338,7 @@ xmlDocPtr rssyl_fetch_feed(const gchar *url, time_t last_update, gchar **title,
 #ifdef RSSYL_DEBUG
 	/* debug mode - get timestamp, add it to returned xmlDoc, and make a copy
 	 * of the fetched feed file */
+	tmptitle = rssyl_strreplace(url, "/", "\\");
 	unixtime_str = g_strdup_printf("%ld", time(NULL) );
 	debugfname = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, RSSYL_DIR,
 			G_DIR_SEPARATOR_S, ".", tmptitle, ".", unixtime_str, NULL);
@@ -347,6 +351,7 @@ xmlDocPtr rssyl_fetch_feed(const gchar *url, time_t last_update, gchar **title,
 	xmlSetProp(rnode, "fetched", unixtime_str);
 	g_free(unixtime_str);
 	g_free(debugfname);
+	g_free(tmptitle);
 #endif	/* RSSYL_DEBUG */
 
 	g_unlink(template);
@@ -554,14 +559,11 @@ gchar *rssyl_format_string(gchar *str, gboolean replace_html, gboolean replace_r
 		res = g_strdup(str);
 
 	if (replace_returns) {
-		tmp = rssyl_strreplace(res, "\n", "");
-		g_free(res);
-		res = tmp;
+		g_strdelimit(res, "\r\n", ' ');
 	}
 
-	tmp = rssyl_strreplace(res, "\t", " ");
-	g_free(res);
-	res = tmp;
+	g_strdelimit(res, "\t", ' ');
+
 	while (strstr(res, "  ")) {
 		tmp = rssyl_strreplace(res, "  ", " ");
 		g_free(res);
@@ -637,6 +639,7 @@ static RSSylFeedItem *rssyl_parse_folder_item_file(gchar *path)
 	fitem->link = NULL;
 	fitem->text = NULL;
 	fitem->id = NULL;
+	fitem->id_is_permalink = FALSE;
 	fitem->realpath = g_strdup(path);
 
 	parsing_headers = TRUE;
@@ -659,7 +662,7 @@ static RSSylFeedItem *rssyl_parse_folder_item_file(gchar *path)
 
 				/* Author */
 				if( !strcmp(line[0], "From") ) {
-					fitem->author = rssyl_format_string(g_strdup(line[1]), TRUE, TRUE);
+					fitem->author = g_strdup(line[1]);
 					debug_print("RSSyl: got author '%s'\n", fitem->author);
 					started_author = TRUE;
 				}
@@ -672,14 +675,14 @@ static RSSylFeedItem *rssyl_parse_folder_item_file(gchar *path)
 
 				/* Title */
 				if( !strcmp(line[0], "Subject") ) {
-					fitem->title = rssyl_format_string(g_strdup(line[1]), TRUE, TRUE);
+					fitem->title = g_strdup(line[1]);
 					debug_print("RSSyl: got title '%s'\n", fitem->title);
 					started_subject = TRUE;
 				}
 
 				/* Link */
 				if( !strcmp(line[0], "X-RSSyl-URL") ) {
-					fitem->link = rssyl_format_string(g_strdup(line[1]), FALSE, FALSE);
+					fitem->link = g_strdup(line[1]);
 					debug_print("RSSyl: got link '%s'\n", fitem->link);
 					started_link = TRUE;
 				}
@@ -693,12 +696,12 @@ static RSSylFeedItem *rssyl_parse_folder_item_file(gchar *path)
 				}
 
 				if( !strcmp(line[0], "X-RSSyl-Comments") ) {
-					fitem->comments_link = rssyl_format_string(g_strdup(line[1]), FALSE, FALSE);
+					fitem->comments_link = g_strdup(line[1]);
 					debug_print("RSSyl: got clink '%s'\n", fitem->comments_link);
 					started_clink = TRUE;
 				}
 				if( !strcmp(line[0], "X-RSSyl-Parent") ) {
-					fitem->parent_link = rssyl_format_string(g_strdup(line[1]), FALSE, FALSE);
+					fitem->parent_link = g_strdup(line[1]);
 					debug_print("RSSyl: got plink '%s'\n", fitem->parent_link);
 					started_plink = TRUE;
 				}
@@ -706,27 +709,27 @@ static RSSylFeedItem *rssyl_parse_folder_item_file(gchar *path)
 				gchar *tmp = NULL;
 				/* continuation line */
 				if (started_author) {
-					tmp = rssyl_format_string(g_strdup_printf("%s %s", fitem->author, lines[i]+1), TRUE, TRUE);
+					tmp = g_strdup_printf("%s %s", fitem->author, lines[i]+1);
 					g_free(fitem->author);
 					fitem->author = tmp;
 					debug_print("RSSyl: updated author to '%s'\n", fitem->author);
 				} else if (started_subject) {
-					tmp = rssyl_format_string(g_strdup_printf("%s %s", fitem->title, lines[i]+1), TRUE, TRUE);
+					tmp = g_strdup_printf("%s %s", fitem->title, lines[i]+1);
 					g_free(fitem->title);
 					fitem->title = tmp;
 					debug_print("RSSyl: updated title to '%s'\n", fitem->title);
 				} else if (started_link) {
-					tmp = rssyl_format_string(g_strdup_printf("%s%s", fitem->link, lines[i]+1), FALSE, FALSE);
+					tmp = g_strdup_printf("%s%s", fitem->link, lines[i]+1);
 					g_free(fitem->link);
 					fitem->link = tmp;
 					debug_print("RSSyl: updated link to '%s'\n", fitem->link);
 				} else if (started_clink) {
-					tmp = rssyl_format_string(g_strdup_printf("%s%s", fitem->comments_link, lines[i]+1), FALSE, FALSE);
+					tmp = g_strdup_printf("%s%s", fitem->comments_link, lines[i]+1);
 					g_free(fitem->comments_link);
 					fitem->comments_link = tmp;
 					debug_print("RSSyl: updated comments_link to '%s'\n", fitem->comments_link);
 				} else if (started_plink) {
-					tmp = rssyl_format_string(g_strdup_printf("%s%s", fitem->parent_link, lines[i]+1), FALSE, FALSE);
+					tmp = g_strdup_printf("%s%s", fitem->parent_link, lines[i]+1);
 					g_free(fitem->parent_link);
 					fitem->parent_link = tmp;
 					debug_print("RSSyl: updated comments_link to '%s'\n", fitem->parent_link);
@@ -1070,12 +1073,14 @@ void rssyl_parse_feed(xmlDocPtr doc, RSSylFolderItem *ritem, gchar *parent)
 gboolean rssyl_add_feed_item(RSSylFolderItem *ritem, RSSylFeedItem *fitem)
 {
 	MsgFlags *flags;
-	gchar *template;
-	gchar tmp[512];
-	gint d, fd, dif = 0;
+	gchar *template, *tmpurl, *tmpid;
+	gchar tmp[1024];
+	gint d = -1, fd, dif = 0;
 	FILE *f;
 	RSSylFeedItem *oldfitem = NULL;
 	gchar *meta_charset = NULL;
+	gboolean err = FALSE;
+
 	g_return_val_if_fail(ritem != NULL, FALSE);
 	g_return_val_if_fail(ritem->item.path != NULL, FALSE);
 	g_return_val_if_fail(fitem != NULL, FALSE);
@@ -1115,7 +1120,7 @@ gboolean rssyl_add_feed_item(RSSylFolderItem *ritem, RSSylFeedItem *fitem)
 
 	if( fitem->date != 0 ) {
 		gchar *tmpdate = createRFC822Date(&fitem->date);
-		fprintf(f, "Date: %s\n", tmpdate );
+		err |= (fprintf(f, "Date: %s\n", tmpdate ) < 0);
 		g_free(tmpdate);
 	}
 
@@ -1123,54 +1128,62 @@ gboolean rssyl_add_feed_item(RSSylFolderItem *ritem, RSSylFeedItem *fitem)
 		if (g_utf8_validate(fitem->author, -1, NULL)) {
 			conv_encode_header_full(tmp, 511, fitem->author, 
 				strlen("From: "), TRUE, CS_UTF_8);
-			fprintf(f, "From: %s\n", tmp);
+			err |= (fprintf(f, "From: %s\n", tmp) < 0);
 		} else
-			fprintf(f, "From: %s\n", fitem->author);
+			err |= (fprintf(f, "From: %s\n", fitem->author) < 0);
 	} 
 
 	if( fitem->title ) {
 		if (g_utf8_validate(fitem->title, -1, NULL)) {
-			conv_encode_header_full(tmp, 511, fitem->title, 
+			conv_encode_header_full(tmp, 1023, fitem->title, 
 				strlen("Subject: "), FALSE, CS_UTF_8);
-			fprintf(f, "Subject: %s\n", rssyl_format_string(tmp, TRUE, TRUE));
+			err |= (fprintf(f, "Subject: %s\n", tmp) < 0);
 		} else
-			fprintf(f, "Subject: %s\n", rssyl_format_string(tmp, TRUE, TRUE));
+			err |= (fprintf(f, "Subject: %s\n", tmp) < 0);
 	}
 
-	if( fitem->link ) {
-		fprintf(f, "X-RSSyl-URL: %s\n", fitem->link);
-		fprintf(f, "Message-ID: <%s>\n", fitem->link);
+	if( (tmpurl = fitem->link) == NULL ) {
+		if( fitem->id != NULL && fitem->id_is_permalink )
+			tmpurl = fitem->id;
 	}
+	if( tmpurl != NULL )
+		err |= (fprintf(f, "X-RSSyl-URL: %s\n", tmpurl) < 0);
+
+	if( (tmpid = fitem->id) == NULL )
+		tmpid = fitem->link;
+	if( tmpid != NULL )
+		err |= (fprintf(f, "Message-ID: <%s>\n", tmpid) < 0);
+
 	if( fitem->comments_link ) {
-		fprintf(f, "X-RSSyl-Comments: %s\n", fitem->comments_link);
+		err |= (fprintf(f, "X-RSSyl-Comments: %s\n", fitem->comments_link) < 0);
 	}
 	if( fitem->parent_link) {
-		fprintf(f, "X-RSSyl-Parent: %s\n", fitem->parent_link);
-		fprintf(f, "References: <%s>\n", fitem->parent_link);
+		err |= (fprintf(f, "X-RSSyl-Parent: %s\n", fitem->parent_link) < 0);
+		err |= (fprintf(f, "References: <%s>\n", fitem->parent_link) < 0);
 	}
 
 #ifdef RSSYL_DEBUG
 	if( fitem->debug_fetched != -1 ) {
-		fprintf(f, "X-RSSyl-Debug-Fetched: %ld\n", fitem->debug_fetched);
+		err |= (fprintf(f, "X-RSSyl-Debug-Fetched: %ld\n", fitem->debug_fetched) < 0);
 	}
 #endif	/* RSSYL_DEBUG */
 
 	if (fitem->text && g_utf8_validate(fitem->text, -1, NULL)) {
 		/* if it passes UTF-8 validation, specify it. */
-		fprintf(f, "Content-Type: text/html; charset=UTF-8\n\n");
+		err |= (fprintf(f, "Content-Type: text/html; charset=UTF-8\n\n") < 0);
 		meta_charset = g_strdup("<meta http-equiv=\"Content-Type\" "
 			       "content=\"text/html; charset=UTF-8\">");
 	} else {
 		/* make sure Claws Mail displays it as html */
-		fprintf(f, "Content-Type: text/html\n\n");
+		err |= (fprintf(f, "Content-Type: text/html\n\n") < 0);
 	}
 
-	if( fitem->link )
-		fprintf(f, "<p>URL: <a href=\"%s\">%s</a></p>\n<br>\n",
-				fitem->link, fitem->link);
+	if( tmpurl )
+		err |= (fprintf(f, "<p>URL: <a href=\"%s\">%s</a></p>\n<br>\n",
+				tmpurl, tmpurl) < 0);
 
 	if( fitem->text )
-		fprintf(f, "<html><head>"
+		err |= (fprintf(f, "<html><head>"
 				"%s\n"
 				"<base href=\"%s\">\n"
 			        "</head><body>\n"
@@ -1179,28 +1192,45 @@ gboolean rssyl_add_feed_item(RSSylFolderItem *ritem, RSSylFeedItem *fitem)
 				RSSYL_TEXT_END"\n\n",
 				meta_charset ? meta_charset:"",
 				fitem->link,
-				fitem->text, (fitem->text ? "\n" : "") );
+				fitem->text, (fitem->text ? "\n" : "") ) < 0);
 
 	g_free(meta_charset);
 
+	if( fitem->media ) {
+		if( fitem->media->size > 0 ) {
+			tmpid = g_strdup_printf(ngettext("%ld byte", "%ld bytes",
+						fitem->media->size), fitem->media->size);
+		} else {
+			tmpid = g_strdup(_("size unknown"));
+		}
+
+		fprintf(f, "<p><a href=\"%s\">Attached media file</a> [%s] (%s)</p>\n",
+				fitem->media->url, fitem->media->type, tmpid);
+		
+		g_free(tmpid);
+	}
+
 	if( fitem->media )
-		fprintf(f, "<p><a href=\"%s\">Attached media file</a> [%s] (%ld bytes)</p>\n",
-				fitem->media->url, fitem->media->type, fitem->media->size);
+		err |= (fprintf(f,
+					"<p><a href=\"%s\">Attached media file</a> [%s] (%ld bytes)</p>\n",
+				fitem->media->url, fitem->media->type, fitem->media->size) < 0);
 
-	fprintf(f, "</body></html>\n");
+	err |= (fprintf(f, "</body></html>\n") < 0);
 
-	fclose(f);
+	err |= (fclose(f) == EOF);
 
 	flags->perm_flags = MSG_NEW | MSG_UNREAD;
 	flags->tmp_flags = 0;
 
-	g_return_val_if_fail(template != NULL, FALSE);
-	d = folder_item_add_msg(&ritem->item, template, flags, TRUE);
+	if (!err) {
+		g_return_val_if_fail(template != NULL, FALSE);
+		d = folder_item_add_msg(&ritem->item, template, flags, TRUE);
+	}
 	g_free(template);
+	
+	debug_print("RSSyl: folder_item_add_msg(): %d, err %d\n", d, err);
 
-	debug_print("RSSyl: folder_item_add_msg(): %d\n", d);
-
-	return TRUE;
+	return err ? FALSE:TRUE;
 }
 
 MsgInfo *rssyl_parse_feed_item_to_msginfo(gchar *file, MsgFlags flags,
@@ -1448,7 +1478,7 @@ static RSSylFolderItem *rssyl_find_feed_by_url(gchar *url)
 	return ritem;
 }
 
-gboolean rssyl_subscribe_new_feed(FolderItem *parent, const gchar *url, 
+FolderItem *rssyl_subscribe_new_feed(FolderItem *parent, const gchar *url, 
 				  gboolean verbose)
 {
 	gchar *title;
@@ -1458,8 +1488,8 @@ gboolean rssyl_subscribe_new_feed(FolderItem *parent, const gchar *url,
 	gchar *myurl = NULL;
 	gchar *error = NULL;
 
-	g_return_val_if_fail(parent != NULL, FALSE);
-	g_return_val_if_fail(url != NULL, FALSE);
+	g_return_val_if_fail(parent != NULL, NULL);
+	g_return_val_if_fail(url != NULL, NULL);
 
 	myurl = g_strdup(url);
 
@@ -1474,27 +1504,37 @@ gboolean rssyl_subscribe_new_feed(FolderItem *parent, const gchar *url,
 		if (verbose)
 			alertpanel_error(_("You are already subscribed to this feed."));
 		g_free(myurl);
-		return FALSE;
+		return NULL;
 	}
-
+	
+	main_window_cursor_wait(mainwindow_get_mainwindow());
 	doc = rssyl_fetch_feed(myurl, -1, &title, &error);
+	main_window_cursor_normal(mainwindow_get_mainwindow());
 	if( !title ) {
-		if (verbose)
-			alertpanel_error(_("Couldn't fetch URL '%s':\n%s"), myurl, error ? error:_("Unknown error"));
-		else
+		if (verbose) {
+			gchar *tmp;
+			tmp = g_markup_printf_escaped(_("Couldn't fetch URL '%s':\n%s"),
+						myurl, error ? error:_("Unknown error"));
+			alertpanel_error("%s", tmp);
+			g_free(tmp);
+		} else
 			log_error(LOG_PROTOCOL, _("Couldn't fetch URL '%s':\n%s\n"), myurl, error ? error:_("Unknown error"));
 		g_free(myurl);
 		g_free(error);
-		return FALSE;
+		return NULL;
 	}
 	g_free(error);
 
 	new_item = folder_create_folder(parent, title);
 	if( !new_item ) {
-		if (verbose)
+		if (verbose) {
+			gchar *tmp;
+			tmp = g_markup_printf_escaped("%s", title);
 			alertpanel_error(_("Can't subscribe feed '%s'."), title);
+			g_free(tmp);
+		}
 		g_free(myurl);
-		return FALSE;
+		return NULL;
 	}
 
 	ritem = (RSSylFolderItem *)new_item;
@@ -1521,7 +1561,7 @@ gboolean rssyl_subscribe_new_feed(FolderItem *parent, const gchar *url,
 	rssyl_start_refresh_timeout(ritem);
 
 	folder_item_scan(new_item);
-	return TRUE;
+	return new_item;
 }
 
 static gint rssyl_expire_sort_func(RSSylFeedItem *a, RSSylFeedItem *b)
