@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -26,11 +26,11 @@
 
 #include <glib.h>
 
-#include "common/utils.h"
 #include "prefs.h"
 #include "prefs_gtk.h"
 #include "prefswindow.h"
 #include "gettext.h"
+#include "common/utils.h"
 
 #include "vcal_prefs.h"
 
@@ -629,11 +629,13 @@ static void vcal_prefs_save_func(PrefsPage * _page)
 		prefs_file_close_revert(pfile);
 		return;
 	}
-	fprintf(pfile->fp, "\n");
-
-	prefs_file_close(pfile);
+        if (fprintf(pfile->fp, "\n") < 0) {
+		FILE_OP_ERROR(rcpath, "fprintf");
+		prefs_file_close_revert(pfile);
+	} else
+	        prefs_file_close(pfile);
 	
-	vcal_folder_export();
+	vcal_folder_export(NULL);
 }
 
 void vcal_prefs_init(void)
